package com.parse;

/**
 * A GetDataCallback is used to run code after a {@link ParseFile} fetches its data on a background thread.
 * <p>
 * The easiest way to use a GetDataCallback is through an anonymous inner class. Override the
 * <code>done</code> function to specify what the callback should do after the fetch is complete. The
 * <code>done</code> function will be run in the UI thread, while the fetch happens in a background thread.
 * This ensures that the UI does not freeze while the fetch happens. 
 * <p>
 * 
 * file.getDataInBackground(new GetDataCallback() {					
 *		public void done(byte[] data, ParseException e) {
 *			// ...
 *		}
 * });
 * 
 * 
 */
public abstract class GetDataCallback extends ParseCallback<byte[]> {
	public abstract void done(byte[] data, ParseException e);
	
	@Override
	final void internalDone(byte[] returnValue, ParseException e) {
		done(returnValue, e);
	}
}
