package com.parse;


/**
 * A ParseCallback is used to do something after a background task completes.
 * End users will use a specific subclass of ParseCallback.
 */
abstract class ParseCallback<T> {
	public ParseCallback() {}

	/**
	 * done(returnValue, e) must be overridden when you are doing a background operation.
	 * It is called when the background operation completes.
	 * <p>
	 * If the operation is successful, returnValue will contain the results and the exception
	 * will be null.
	 * <p>
	 * If the operation was unsuccessful, returnValue will be null and the exception will
	 * contain information about the operation failure.
	 * <p>
	 * @param returnValue The results of the operation.
	 * @param e The ParseException that was thrown by the operation, if there was any.
	 */	
	abstract void internalDone(T returnValue, ParseException e);
}
