package com.parse;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONStringer;
import org.json.JSONTokener;

import android.util.Log;

import com.parse.signpost.OAuthConsumer;
import com.parse.signpost.commonshttp.CommonsHttpOAuthConsumer;
import com.parse.signpost.exception.OAuthCommunicationException;
import com.parse.signpost.exception.OAuthExpectationFailedException;
import com.parse.signpost.exception.OAuthMessageSignerException;

/**
 * A helper object to send requests to the server.
 */
class ParseCommand {
	// private static final String TAG = "com.parse.ParseCommand";
	
	String op;	
	JSONObject params;
	
	private HttpClient client;
	private HttpPost post;
	
	private boolean cancelled = false; // Whether the request has been cancelled
	private boolean retryEnabled = false; // Whether retries are enabled
	private int attemptsMade = 0; // Number of connections attempted
	private long delay = 0; // Delay for the next retry (in milliseconds)
	
	private static int SOCKET_TIMEOUT = 10000;
	private static int CONNECTION_TIMEOUT = 10000;
	
	ParseCommand(String theOp, HttpClient httpClient) {
		op = theOp;
		params = new JSONObject();
		client = httpClient;
	}
	
	ParseCommand(String theOp) {
		op = theOp;
		params = new JSONObject();

		HttpParams httpParameters = new BasicHttpParams();
		HttpConnectionParams.setConnectionTimeout(httpParameters, CONNECTION_TIMEOUT);
		HttpConnectionParams.setSoTimeout(httpParameters, SOCKET_TIMEOUT);

		client = new DefaultHttpClient(httpParameters);
	}
	
	// The various put functions add to params.
	void put(String key, String value) {
		try {
			params.put(key, value);
		} catch (JSONException e) {
			throw new RuntimeException(e.getMessage());
		}
	}
	
	void put(String key, int value) {
		try {
			params.put(key, value);
		} catch (JSONException e) {
			throw new RuntimeException(e.getMessage());
		}		
	}
	
	void put(String key, JSONArray value) {
		try {
			params.put(key, value);
		} catch (JSONException e) {
			throw new RuntimeException(e.getMessage());
		}		
	}
	
	void put(String key, JSONObject value) {
		try {
			params.put(key, value);
		} catch (JSONException e) {
			throw new RuntimeException(e.getMessage());
		}		
	}
	
	void setOp(String theOp) {
		op = theOp;
	}
	

	// Converts the whole command to a json object, suitable for
	// 'multi' requests.
	JSONObject toJSONObject() {
		try {
			JSONObject answer = new JSONObject();
			answer.put("op", op);
			answer.put("params", params);
			return answer;
		} catch (JSONException e) {
			throw new RuntimeException(e.getMessage());
		}
	}
	
	// Creates a somewhat-readable string that uniquely identifies this command.
	String getCacheKey() {
		String json;
		try {
			json = toDeterministicString(params);
		} catch (JSONException e) {
			throw new RuntimeException(e.getMessage());
		}

		// Include the current user in the cache in order to avoid mixing permissions
		ParseUser currentUser = ParseUser.getCurrentUser();
		String sessionToken = null;
		if (currentUser != null) {
			sessionToken = currentUser.getSessionToken();
			if (sessionToken != null) {
				json += sessionToken;
			}
		}
		
		return ("ParseCommand." + op + "." + ParseObject.API_VERSION + "."
				+ Math.abs(json.hashCode()));
	}
	
	// Encodes the object to JSON, but ensures that JSONObjects and nested JSONObjects
	// are encoded with keys in alphabetical order.
	static String toDeterministicString(Object o) throws JSONException {
		JSONStringer stringer = new JSONStringer();
		addToStringer(stringer, o);
		return stringer.toString();
	}
	
	// Uses the provided JSONStringer to encode this object to JSON, but ensures that
	// JSONObjects and nested JSONObjects are encoded with keys in alphabetical order.
	static void addToStringer(JSONStringer stringer, Object o) throws JSONException {
		if (o instanceof JSONObject) {
			stringer.object();
			JSONObject object = (JSONObject) o;
			@SuppressWarnings("unchecked")
			Iterator<String> keyIterator = object.keys();
			ArrayList<String> keys = new ArrayList<String>();
			while (keyIterator.hasNext()) {
				keys.add(keyIterator.next());
			}
			Collections.sort(keys);
			
			for (String key : keys) {
				stringer.key(key);
				addToStringer(stringer, object.opt(key));
			}
			
			stringer.endObject();
			return;
		}

		if (o instanceof JSONArray) {
			JSONArray array = (JSONArray) o;
			stringer.array();
			for (int i = 0; i < array.length(); ++i) {
				addToStringer(stringer, array.get(i));
			}
			stringer.endArray();
			return;
		}

		stringer.value(o);
	}
	
	// Helper to perform a request without caching.
	Object perform() throws ParseException {
		return perform(false);
	}
	
	// Helper to give decent errors
	ParseException connectionFailed(String message, Exception e) {
		return new ParseException(ParseException.CONNECTION_FAILED,
				message + ": " + e.getClass().getName() + ": " + e.getMessage());
	}
	
	void preparePost() throws ParseException {
		// Convert params to url format
		List<NameValuePair> urlParams = new ArrayList<NameValuePair>();
		@SuppressWarnings("unchecked")
		Iterator<String> keys = params.keys();
		while (keys.hasNext()) {
			String key = keys.next();
			try {
				urlParams.add(new BasicNameValuePair(key, params.get(key).toString()));
			} catch (JSONException e) {
				throw new RuntimeException(e.getMessage());
			}
		}
		
		// a is for android
		urlParams.add(new BasicNameValuePair("v", "a" + ParseObject.VERSION_NAME));

		// add uuid
		urlParams.add(new BasicNameValuePair("uuid", UUID.randomUUID().toString()));
		
		// Pass the session token on every request
		ParseUser currentUser = ParseUser.getCurrentUser();
		if (currentUser != null && currentUser.getSessionToken() != null) {
			urlParams.add(new BasicNameValuePair(
					"session_token",
					currentUser.getSessionToken()));			
		}

		// Construct a post request
		String url = String.format("%s/%s/%s", ParseObject.server,
				ParseObject.API_VERSION, op);
		
		post = new HttpPost(url);
		try {
			post.setEntity(new UrlEncodedFormEntity(urlParams, "UTF-8"));
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e.getMessage());
		}
			
		// Authenticate the request
		OAuthConsumer consumer = new CommonsHttpOAuthConsumer(
					Parse.applicationId, Parse.clientKey); 
		consumer.setTokenWithSecret(null, "");
		try {
			consumer.sign(post);
		} catch (OAuthMessageSignerException e) {
			throw new ParseException(ParseException.NOT_INITIALIZED, e.getMessage());
		} catch (OAuthExpectationFailedException e) {
			throw new ParseException(ParseException.NOT_INITIALIZED, e.getMessage());
		} catch (OAuthCommunicationException e) {
			throw new ParseException(ParseException.NOT_INITIALIZED, e.getMessage());
		}
	}
	
	/**
	 * Helper function to send requests to the server.
	 * Client code should not need to use this function.
	 * @param saveToCache Whether to save the results of the command to the cache.
	 * @return The JSON response from the server.
	 * @throws ParseException Throws an exception if the server returns an error or is inaccessible.
	 */
	Object perform(boolean saveToCache) throws ParseException {
		Parse.checkInit();

		// Set up the HttpPost just once.
		if (post == null) {
			preparePost();
		}
		
		JSONObject result = null;

		try {
			result = sendRequest(client, post);
		} catch (ParseException e) {
			attemptsMade++;
			
			// TODO(ilya): Make sure we don't retry auth failures.
			if (retryEnabled && attemptsMade < 5) {
				Log.d("com.parse.ParseCommand", "Fetch failed. Waiting " + delay + " milliseconds before attempt #" + (attemptsMade + 1));
				try {
					Thread.sleep(delay);
				} catch (InterruptedException e1) {
				}
				
				calculateNextDelay();
				return perform(saveToCache);
			} else if (!cancelled) {
				throw e;
			}
		}
		
		// Parse the response
		Object answer;
		try {
			if (result.has("error")) {
				throw new ParseException(
						result.getInt("code"),
						result.getString("error"));
			}
			answer = result.get("result");
		} catch (JSONException e) {
			throw connectionFailed("corrupted json", e);
		}
		
		if (saveToCache) {
			Parse.saveToKeyValueCache(getCacheKey(), answer.toString());
		}
		return answer;
	}
	
	private JSONObject sendRequest(HttpClient client, HttpPost post) throws ParseException {
		JSONObject result;
		
		// Send the request
		HttpResponse rawResponse;
		String json;
		try {
			rawResponse = client.execute(post);
			BufferedReader reader = new BufferedReader(
					new InputStreamReader(rawResponse.getEntity().getContent(), "UTF-8"),
					8 * 1024);
			json = reader.readLine();
		} catch (ClientProtocolException e) {
			throw connectionFailed("bad protocol", e);
		} catch (IOException e) {
			throw connectionFailed("i/o failure", e);
		} 
		
		try {
			JSONTokener tokener = new JSONTokener(json);
			result = new JSONObject(tokener);
		} catch (JSONException e) {
			throw connectionFailed("bad json response", e);
		}
		
		return result;
	}
	
	private void calculateNextDelay() {
		delay = delay * 2;
	}

	public void enableRetrying() {
	    retryEnabled = true;
	    
	    // Set the initial delay to something between 1 and 2 seconds. We want it to be
	    // random so that clients that fail simultaneously don't retry on simultaneous
	    // intervals.
	    delay = 1000 + (long)(1000 * Math.random());
	}
	
	public void cancel() {
		if (post != null) {
			post.abort();
		}
		cancelled = true;
	}
}
