package com.parse;

import java.util.Random;

import org.json.JSONObject;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;

// A callback for standard push messages that we just want to display a normal
// notification for, and then send the user to an activity.
// Don't change the name of this class, because callback serialization depends on
// it via reflection.
public class StandardPushCallback extends PushCallback {
	// The total number of notifications sent by a StandardPushCallback
	// during this process's lifetime.
	static int totalNotifications = 0;
	
	// A global flag to disable notification-showing.
	static boolean disableNotifications = false;
	
	private Bundle makeBundle() {
		Bundle bundle = new Bundle();
		bundle.putString("com.parse.Data", pushData.toString());
		bundle.putString("com.parse.Channel", channel);
		return bundle;
	}
	
	public void run() {
		++totalNotifications;
		if (disableNotifications) {
			return;
		}
		
		// Avoid null dereferencing
		if (pushData == null) {
			pushData = new JSONObject();
		}

		if (pushData.has("action")) {
			// Fire off a broadcast intent
			Intent broadcastIntent = new Intent();
			broadcastIntent.putExtras(makeBundle());
			broadcastIntent.setAction(pushData.optString("action"));
			service.getBaseContext().sendBroadcast(broadcastIntent);
			if (!pushData.has("alert") && !pushData.has("title")) {
				// If the user provides action but not alert or title, we
				// shouldn't show a notification in the tray.
				return;
			}
		}		
		
		// Analyze the payload
		String text = pushData.optString("alert", "Notification received.");
		String title = pushData.optString(
				"title", localData.optString("appName"));
		
		// Construct the Intents
		ComponentName componentName = new ComponentName(
				localData.optString("activityPackage"),
				localData.optString("activityClass"));
		Intent intent = new Intent();
		intent.setComponent(componentName);
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		intent.putExtras(makeBundle());
		PendingIntent contentIntent = PendingIntent.getActivity(
				service, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);

		// Construct a Notification
		long when = System.currentTimeMillis();
		Notification notif = new Notification(localData.optInt("icon"), text, when);
		notif.flags |= Notification.FLAG_AUTO_CANCEL;
		notif.defaults |= Notification.DEFAULT_ALL;
		notif.setLatestEventInfo(service, title, text, contentIntent);
				
		// Fire off the notification
		NotificationManager nm = (NotificationManager) service.getSystemService(
				Context.NOTIFICATION_SERVICE);
		Random rand = new Random();
		// Pick an id that probably won't overlap anything
		int notificationId = 7000000 + rand.nextInt(1000000);
			
		try {
			nm.notify(notificationId, notif);
		} catch (SecurityException e) {
			// Some phones throw an exception for unapproved vibration
			notif.defaults = Notification.DEFAULT_LIGHTS | Notification.DEFAULT_SOUND;
			nm.notify(notificationId, notif);
		}
	}
}
